@@ -22,6 +22,7 @@ import gobject
 import logging
 import weakref
 import time
+import re
 
 import dbus
 import telepathy
@@ -65,7 +66,7 @@ class ButterflyTextChannel(
             'SupportedContentTypes': lambda: ["text/plain"] ,
             'MessagePartSupportFlags': lambda: 0,
             'DeliveryReportingSupport': lambda: telepathy.DELIVERY_REPORTING_SUPPORT_FLAG_RECEIVE_FAILURES,
-            'PendingMessages': lambda: self._pending_messages2
+            'PendingMessages': lambda: dbus.Array(self._pending_messages2.values(), signature='aa{sv}')
             })
 
         self._add_immutables({
@@ -156,19 +157,19 @@ class ButterflyTextChannel(
 
     def _signal_text_received(self, id, timestamp, sender, type, flags, sender_nick, text):
         self.Received(id, timestamp, sender, type, flags, text)
-        headers = {'message-received' : timestamp,
-                   'pending-message-id' : int(id),
-                   'message-sender' : int(sender),
-                   'message-type' : type
-                  }
+        headers = dbus.Dictionary({dbus.String('message-received') : dbus.UInt64(timestamp),
+                   dbus.String('pending-message-id') : dbus.UInt32(id),
+                   dbus.String('message-sender') : dbus.UInt32(sender),
+                   dbus.String('message-type') : dbus.UInt32(type)
+                  }, signature='sv')
 
         if sender_nick not in (None, ''):
-            headers['sender-nickname'] = sender_nick
+            headers[dbus.String('sender-nickname')] = dbus.String(sender_nick)
 
-        body = {'content-type': 'text/plain',
-                'content': text
-               }
-        message = [headers, body]
+        body = dbus.Dictionary({dbus.String('content-type'): dbus.String('text/plain'),
+                dbus.String('content'): dbus.String(text)
+               }, signature='sv')
+        message = dbus.Array([headers, body], signature='a{sv}')
         self.MessageReceived(message)
 
     def SetChatState(self, state):
@@ -217,7 +218,7 @@ class ButterflyTextChannel(
                          out_signature='s', async_callbacks=('_success', '_error'))
     def SendMessage(self, message, flags, _success, _error):
         headers = message.pop(0)
-        message_type = int(headers['message-type'])
+        message_type = int(headers.get('message-type', telepathy.CHANNEL_TEXT_MESSAGE_TYPE_NORMAL))
         if message_type != telepathy.CHANNEL_TEXT_MESSAGE_TYPE_NORMAL:
                 raise telepathy.NotImplemented("Unhandled message type")
         text = None
@@ -243,9 +244,11 @@ class ButterflyTextChannel(
         self.PendingMessagesRemoved(ids)
 
     def ListPendingMessages(self, clear):
-        ids = self._pending_messages2.keys()
-        self._pending_messages2 = {}
-        self.PendingMessagesRemoved(ids)
+        if clear:
+            ids = self._pending_messages2.keys()
+            self._pending_messages2 = {}
+            self.PendingMessagesRemoved(ids)
+
         return telepathy.server.ChannelTypeText.ListPendingMessages(self, clear)
 
     # Redefine GetSelfHandle since we use our own handle
@@ -287,7 +290,9 @@ class ButterflyTextChannel(
                 sender.account, sender.network_id)
         type = telepathy.CHANNEL_TEXT_MESSAGE_TYPE_NORMAL
         logger.info("User %s sent a message" % unicode(handle))
-        self._signal_text_received(id, timestamp, handle, type, 0, message.display_name, message.content)
+        content = re.sub('\r\n', '\n', message.content)
+        content = re.sub('\r', '\n', content)
+        self._signal_text_received(id, timestamp, handle, type, 0, message.display_name, content)
         self._recv_id += 1
 
     # papyon.event.ConversationEventInterface
@@ -302,5 +307,5 @@ class ButterflyTextChannel(
     @dbus.service.signal(telepathy.CHANNEL_INTERFACE_MESSAGES, signature='aa{sv}')
     def MessageReceived(self, message):
         id = message[0]['pending-message-id']
-        self._pending_messages2[id] = message
+        self._pending_messages2[id] = dbus.Array(message, signature='a{sv}')
 
