@@ -28,8 +28,8 @@ import papyon.event
 from butterfly.presence import ButterflyPresence
 from butterfly.aliasing import ButterflyAliasing
 from butterfly.avatars import ButterflyAvatars
-from butterfly.handle import ButterflyHandleFactory
 from butterfly.capabilities import ButterflyCapabilities
+from butterfly.handle import ButterflyHandleFactory, network_to_extension
 from butterfly.contacts import ButterflyContacts
 from butterfly.channel_manager import ButterflyChannelManager
 from butterfly.mail_notification import ButterflyMailNotification
@@ -70,11 +70,13 @@ class ButterflyConnection(telepathy.server.Connection,
             'https-proxy-server' : 's',
             'https-proxy-port' : 'q',
             'https-proxy-username' : 's',
-            'https-proxy-password' : 's'
+            'https-proxy-password' : 's',
+            'http-method' : 'b',
             }
     _parameter_defaults = {
             'server' : 'messenger.hotmail.com',
-            'port' : 1863
+            'port' : 1863,
+            'http-method' : False
             }
 
     def __init__(self, manager, parameters):
@@ -82,19 +84,27 @@ class ButterflyConnection(telepathy.server.Connection,
 
         try:
             account = unicode(parameters['account'])
-            server = (parameters['server'].encode('utf-8'), parameters['port'])
+            self._server = (parameters['server'].encode('utf-8'), parameters['port'])
 
             # Build the proxies configurations
-            proxies = {}
+            self._proxies = {}
             proxy = build_proxy_infos(parameters, 'http')
             if proxy is not None:
-                proxies['http'] = proxy
+                self._proxies['http'] = proxy
             proxy = build_proxy_infos(parameters, 'https')
             if proxy is not None:
-                proxies['https'] = proxy
+                self._proxies['https'] = proxy
+
+            self._suggested_proxies = []
+
+            # If the HTTP proxy parameters have been set, don't try any
+            # others proxies automatically.
+            if 'http' not in self._proxies:
+                self._fill_suggested_proxies()
+                self._use_next_proxy()
 
             self._manager = weakref.proxy(manager)
-            self._msn_client = papyon.Client(server, proxies)
+            self._new_client(use_http=parameters['http-method'])
             self._account = (parameters['account'].encode('utf-8'),
                     parameters['password'].encode('utf-8'))
             self._channel_manager = ButterflyChannelManager(self)
@@ -108,10 +118,6 @@ class ButterflyConnection(telepathy.server.Connection,
             ButterflyCapabilities.__init__(self)
             ButterflyContacts.__init__(self)
             ButterflyMailNotification.__init__(self)
-            papyon.event.ClientEventInterface.__init__(self, self._msn_client)
-            papyon.event.InviteEventInterface.__init__(self, self._msn_client)
-            papyon.event.OfflineMessagesEventInterface.__init__(self, self._msn_client)
-
 
             self.set_self_handle(ButterflyHandleFactory(self, 'self'))
 
@@ -125,6 +131,84 @@ class ButterflyConnection(telepathy.server.Connection,
             logger.exception("Failed to create Connection")
             raise
 
+    def _new_client(self, use_http=False):
+        if hasattr(self, '_msn_client') and self._msn_client:
+            self._msn_client.logout()
+            self._msn_client._events_handlers.remove(self)
+
+        if use_http:
+            self._tried_http = True
+            self._msn_client = papyon.Client(self._server, self._proxies,
+                papyon.transport.HTTPPollConnection)
+        else:
+            self._tried_http = False
+            self._msn_client = papyon.Client(self._server, self._proxies)
+
+        papyon.event.ClientEventInterface.__init__(self, self._msn_client)
+        papyon.event.InviteEventInterface.__init__(self, self._msn_client)
+        papyon.event.OfflineMessagesEventInterface.__init__(self, self._msn_client)
+
+    def _fill_suggested_proxies(self):
+        try:
+            import libproxy
+        except ImportError:
+            return
+
+        factory = libproxy.ProxyFactory()
+        proxies = factory.getProxies('http://gateway.messenger.msn.com/')
+
+        # Remove socks proxies that papyon doesn't support.
+        proxies = [p for p in proxies if p.startswith('http://') or p == 'direct://']
+
+        if proxies:
+            self._suggested_proxies = proxies
+
+    def _use_next_proxy(self):
+        if not self._suggested_proxies:
+            return False
+
+        # Use the first one.
+        proxy = self._suggested_proxies.pop(0)
+
+        if proxy == 'direct://':
+            if 'http' in self._proxies:
+                del self._proxies['http']
+            return True
+
+        # libproxy documentation states:
+        #
+        #  * The format of the returned proxy strings are as follows:
+        #  *   - http://[username:password@]proxy:port
+        #  *   - socks://[username:password@]proxy:port
+        #  *   - direct://
+        #  etc.
+        #
+        # We've already removed every proxy other than http and
+        # direct, and have dealt with direct, so any other element
+        # will be an HTTP proxy:
+
+        proxy = proxy[len('http://'):]
+
+        # Get username and password out.
+        if '@' in proxy:
+            auth, proxy = proxy.split('@')
+            user, password = auth.split(':')
+        else:
+            user = password = None
+
+        server, port = proxy.split(':')
+
+        self._proxies['http'] = \
+            papyon.ProxyInfos(host=server, port=int(port), type='http',
+                user=user, password=password)
+
+        if user:
+            logger.info('Using proxy: http://%s:***@%s:%u' % (user, server, int(port)))
+        else:
+            logger.info('Using proxy: http://%s:%u' % (server, int(port)))
+
+        return True
+
     @property
     def manager(self):
         return self._manager
@@ -148,6 +232,20 @@ class ButterflyConnection(telepathy.server.Connection,
         self.__disconnect_reason = telepathy.CONNECTION_STATUS_REASON_REQUESTED
         self._msn_client.logout()
 
+    def GetInterfaces(self):
+        # The self._interfaces set is only ever touched in ButterflyConnection.__init__,
+        # where connection interfaces are added.
+
+        # The mail notification interface is added then too, but also removed in its
+        # ButterflyMailNotification.__init__ because it might not actually be available.
+        # It is added before the connection status turns to connected, if available.
+
+        # The spec denotes that this method can return a subset of the actually
+        # available interfaces before connected. As the only possible change will
+        # be adding the mail notification interface before connecting, this is fine.
+
+        return self._interfaces
+
     def RequestHandles(self, handle_type, names, sender):
         self.check_connected()
         self.check_handle_type(handle_type)
@@ -155,23 +253,17 @@ class ButterflyConnection(telepathy.server.Connection,
         handles = []
         for name in names:
             if handle_type == telepathy.HANDLE_TYPE_CONTACT:
-                name = name.rsplit('#', 1)
-                contact_name = name[0]
-                if len(name) > 1:
-                    network_id = int(name[1])
-                else:
-                    network_id = papyon.NetworkID.MSN
+                network_id = papyon.NetworkID.MSN
+                for network, extension in network_to_extension.items():
+                    if name.endswith(extension):
+                        name = name[0:-len(extension)]
+                        network_id = network
+                        break
                 contacts = self.msn_client.address_book.contacts.\
-                        search_by_account(contact_name).\
+                        search_by_account(name).\
                         search_by_network_id(network_id)
-
-                if len(contacts) > 0:
-                    contact = contacts[0]
-                    handle = ButterflyHandleFactory(self, 'contact',
-                            contact.account, contact.network_id)
-                else:
-                    handle = ButterflyHandleFactory(self, 'contact',
-                            contact_name, network_id)
+                handle = ButterflyHandleFactory(self, 'contact',
+                        name, network_id)
             elif handle_type == telepathy.HANDLE_TYPE_LIST:
                 handle = ButterflyHandleFactory(self, 'list', name)
             elif handle_type == telepathy.HANDLE_TYPE_GROUP:
@@ -185,13 +277,15 @@ class ButterflyConnection(telepathy.server.Connection,
     def _generate_props(self, channel_type, handle, suppress_handler, initiator_handle=None):
         props = {
             telepathy.CHANNEL_INTERFACE + '.ChannelType': channel_type,
-            telepathy.CHANNEL_INTERFACE + '.TargetHandle': 0 if handle is None else handle.get_id(),
-            telepathy.CHANNEL_INTERFACE + '.TargetHandleType': telepathy.HANDLE_TYPE_NONE if handle is None else handle.get_type(),
+            telepathy.CHANNEL_INTERFACE + '.TargetHandle': handle.get_id(),
+            telepathy.CHANNEL_INTERFACE + '.TargetHandleType': handle.get_type(),
             telepathy.CHANNEL_INTERFACE + '.Requested': suppress_handler
             }
 
         if initiator_handle is not None:
-            props[telepathy.CHANNEL_INTERFACE + '.InitiatorHandle'] = initiator_handle.id
+            if initiator_handle.get_type() is not telepathy.HANDLE_TYPE_NONE:
+                props[telepathy.CHANNEL_INTERFACE + '.InitiatorHandle'] = \
+                        initiator_handle.get_id()
 
         return props
 
@@ -203,8 +297,8 @@ class ButterflyConnection(telepathy.server.Connection,
         self.check_connected()
         channel_manager = self._channel_manager
 
-        if handle_id == 0:
-            handle = None
+        if handle_id == telepathy.HANDLE_TYPE_NONE:
+            handle = telepathy.server.handle.NoneHandle()
         else:
             handle = self.handle(handle_type, handle_id)
         props = self._generate_props(type, handle, suppress_handler)
@@ -254,6 +348,8 @@ class ButterflyConnection(telepathy.server.Connection,
                 self._channel_manager.channel_for_props(props, signal=True)
         elif state == papyon.event.ClientState.OPEN:
             self._populate_capabilities()
+            if self._client.profile.profile['EmailEnabled'] == '1':
+                self.enable_mail_notification_interface()
             self.StatusChanged(telepathy.CONNECTION_STATUS_CONNECTED,
                     telepathy.CONNECTION_STATUS_REASON_REQUESTED)
             presence = self._initial_presence
@@ -268,8 +364,6 @@ class ButterflyConnection(telepathy.server.Connection,
                 self._presence_changed(ButterflyHandleFactory(self, 'self'),
                         self._client.profile.presence,
                         self._client.profile.personal_message)
-            if self._client.profile.profile['EmailEnabled'] == '1':
-                self.enable_mail_notification_interface()
         elif state == papyon.event.ClientState.CLOSED:
             self.StatusChanged(telepathy.CONNECTION_STATUS_DISCONNECTED,
                     self.__disconnect_reason)
@@ -280,7 +374,19 @@ class ButterflyConnection(telepathy.server.Connection,
     # papyon.event.ClientEventInterface
     def on_client_error(self, type, error):
         if type == papyon.event.ClientErrorType.NETWORK:
-            self.__disconnect_reason = telepathy.CONNECTION_STATUS_REASON_NETWORK_ERROR
+            # Only move onto the next proxy if we've not already tried
+            # HTTP and we're in the connecting state. We don't want to
+            # connect to HTTP if we're already connected and we lose
+            # connectivity (see fd.o#26147).
+            if self._status == telepathy.CONNECTION_STATUS_CONNECTING and \
+                    (self._tried_http is False or \
+                   (self._tried_http is True and self._use_next_proxy())):
+                logger.info("Failed to connect, trying HTTP "
+                            "(possibly again with another proxy)")
+                self._new_client(use_http=True)
+                self._msn_client.login(*self._account)
+            else:
+                self.__disconnect_reason = telepathy.CONNECTION_STATUS_REASON_NETWORK_ERROR
         elif type == papyon.event.ClientErrorType.AUTHENTICATION:
             self.__disconnect_reason = telepathy.CONNECTION_STATUS_REASON_AUTHENTICATION_FAILED
         elif type == papyon.event.ClientErrorType.PROTOCOL and \
@@ -292,42 +398,49 @@ class ButterflyConnection(telepathy.server.Connection,
     # papyon.event.InviteEventInterface
     def on_invite_conversation(self, conversation):
         logger.debug("Conversation invite")
-        #FIXME: get rid of this crap and implement group support
-        participants = conversation.participants
-        for p in participants:
-            participant = p
-            break
-        handle = ButterflyHandleFactory(self, 'contact',
-                participant.account, participant.network_id)
+
+        if len(conversation.participants) == 1:
+            p = list(conversation.participants)[0]
+            handle = ButterflyHandleFactory(self, 'contact',
+                    p.account, p.network_id)
+        else:
+            handle = telepathy.server.handle.NoneHandle()
 
         props = self._generate_props(telepathy.CHANNEL_TYPE_TEXT,
             handle, False, initiator_handle=handle)
+
         channel = self._channel_manager.channel_for_props(props,
             signal=True, conversation=conversation)
 
+        if channel._conversation is not conversation:
+            # If we get an existing channel, attach the conversation object to it
+            channel.attach_conversation(conversation)
+
     # papyon.event.InviteEventInterface
     def on_invite_conference(self, call):
-        logger.debug("Call invite")
-        handle = ButterflyHandleFactory(self, 'contact', call.peer.account,
-                call.peer.network_id)
+        logger.debug("Call invite, ignoring it")
+        #logger.debug("Call invite")
+        #handle = ButterflyHandleFactory(self, 'contact', call.peer.account,
+                #call.peer.network_id)
 
-        props = self._generate_props(telepathy.CHANNEL_TYPE_STREAMED_MEDIA,
-                handle, False, initiator_handle=handle)
+        #props = self._generate_props(telepathy.CHANNEL_TYPE_STREAMED_MEDIA,
+                #handle, False, initiator_handle=handle)
 
-        channel = self._channel_manager.channel_for_props(props,
-                signal=True, call=call)
+        #channel = self._channel_manager.channel_for_props(props,
+                #signal=True, call=call)
 
     # papyon.event.InviteEventInterface
     def on_invite_webcam(self, session, producer):
         direction = (producer and "send") or "receive"
-        logger.debug("Invitation to %s webcam" % direction)
+        logger.debug("Invitation to %s webcam, ignoring it" % direction)
+        #logger.debug("Invitation to %s webcam" % direction)
 
-        handle = ButterflyHandleFactory(self, 'contact', session.peer.account,
-                session.peer.network_id)
-        props = self._generate_props(telepathy.CHANNEL_TYPE_STREAMED_MEDIA,
-                handle, False, initiator_handle=handle)
-        channel = self._channel_manager.channel_for_props(props, signal=True,
-                call=session)
+        #handle = ButterflyHandleFactory(self, 'contact', session.peer.account,
+                #session.peer.network_id)
+        #props = self._generate_props(telepathy.CHANNEL_TYPE_STREAMED_MEDIA,
+                #handle, False, initiator_handle=handle)
+        #channel = self._channel_manager.channel_for_props(props, signal=True,
+                #call=session)
 
     # papyon.event.OfflineMessagesEventInterface
     def on_oim_messages_received(self, messages):
@@ -354,7 +467,7 @@ class ButterflyConnection(telepathy.server.Connection,
         self._manager.disconnected(self)
 
 
-def build_proxy_infos(self, parameters, proxy_type='http'):
+def build_proxy_infos(parameters, proxy_type='http'):
     server_key = proxy_type + '-proxy-server'
     port_key = proxy_type + '-proxy-port'
     username_key = proxy_type + '-proxy-username'
